@@ -14,9 +14,10 @@ Connect to SciDB and run a query:
 Download data from SciDB:
 
 >>> db.arrays.foo[:]
-... # doctest: +NORMALIZE_WHITESPACE
-array([(0, (255, 0)), (1, (255, 1)), (2, (255, 2))],
-      dtype=[('i', '<i8'), ('x', [('null', 'u1'), ('val', '<i8')])])
+   i    x
+0  0  0.0
+1  1  1.0
+2  2  2.0
 
 Upload data to SciDB and create an array:
 
@@ -29,12 +30,18 @@ py_..._1
 Run a query with chained operators and download the resulting array:
 
 >>> db.join(ar, 'foo').apply('j', ar.i + 1)[:]
-... # doctest: +NORMALIZE_WHITESPACE
-array([(0, 0, (255, 0), 1), (1, 1, (255, 1), 2), (2, 2, (255, 2), 3)],
-      dtype=[('i', '<i8'),
-             ('x', '<i8'),
-             ('x_1', [('null', 'u1'), ('val', '<i8')]),
-             ('j', '<i8')])
+   i  x  x_1  j
+0  0  0  0.0  1
+1  1  1  1.0  2
+2  2  2  2.0  3
+
+Operators can also be composed:
+
+>>> db.apply(db.join(ar, 'foo'), 'j', ar.i + 1)[:]
+   i  x  x_1  j
+0  0  0  0.0  1
+1  1  1  1.0  2
+2  2  2  2.0  3
 
 Cleanup:
 
@@ -45,7 +52,7 @@ Cleanup:
 Requirements
 ============
 
-SciDB ``16.9`` with Shim
+SciDB ``16.9`` or newer with Shim
 
 Python ``2.7.x``, ``3.4.x``, ``3.5.x``, ``3.6.x`` or newer.
 
@@ -89,14 +96,19 @@ is none is provided.
 Display information about the ``db`` object:
 
 >>> db
-DB('http://localhost:8080', None, None, None, None)
+DB('http://localhost:8080', None, None, None, False, None, False, 256, False)
 
 >>> print(db)
-scidb_url  = http://localhost:8080
-scidb_auth = None
-http_auth  = None
-namespace  = None
-verify     = None
+scidb_url         = http://localhost:8080
+scidb_auth        = None
+http_auth         = None
+verify            = None
+admin             = False
+namespace         = None
+use_arrow         = False
+result_size_limit = 256
+no_ops            = False
+
 
 Advanced Connection
 -------------------
@@ -106,14 +118,27 @@ Provide `Shim <https://github.com/Paradigm4/shim>`_ credentials:
 >>> db = connect(http_auth=('foo', 'bar'))
 
 >>> db
-DB('http://localhost:8080', None, ('foo', PASSWORD_PROVIDED), None, None)
+... # doctest: +NORMALIZE_WHITESPACE
+DB('http://localhost:8080',
+   None,
+   ('foo', PASSWORD_PROVIDED),
+   None,
+   False,
+   None,
+   False,
+   256,
+   False)
 
 >>> print(db)
-scidb_url  = http://localhost:8080
-scidb_auth = None
-http_auth  = ('foo', PASSWORD_PROVIDED)
-namespace  = None
-verify     = None
+scidb_url         = http://localhost:8080
+scidb_auth        = None
+http_auth         = ('foo', PASSWORD_PROVIDED)
+verify            = None
+admin             = False
+namespace         = None
+use_arrow         = False
+result_size_limit = 256
+no_ops            = False
 
 To prompt the user for the password, use:
 
@@ -128,11 +153,15 @@ Use SSL:
 >>> db_ssl = connect('https://localhost:8083', verify=False)
 
 >>> print(db_ssl)
-scidb_url  = https://localhost:8083
-scidb_auth = None
-http_auth  = None
-namespace  = None
-verify     = False
+scidb_url         = https://localhost:8083
+scidb_auth        = None
+http_auth         = None
+verify            = False
+admin             = False
+namespace         = None
+use_arrow         = False
+result_size_limit = 256
+no_ops            = False
 
 See Python `requests <http://docs.python-requests.org/en/master/>`_
 library `SSL Cert Verification
@@ -140,7 +169,7 @@ library `SSL Cert Verification
 #ssl-cert-verification>`_ section for details on the ``verify``
 argument. ``verify=False`` disables SSL certificate
 verification. Warnings about the unverified HTTPS requests are
-displayed. The warnings can be disables as well by either setting
+displayed. The warnings can be disabled as well by either setting
 ``PYTHONWARNINGS="ignore:Unverified HTTPS request"`` in the
 environment before starting Python or by doing:
 
@@ -155,11 +184,36 @@ Use SSL and SciDB credentials:
 ...   'https://localhost:8083', scidb_auth=('foo', 'bar'), verify=False)
 
 >>> print(db_ssl)
-scidb_url  = https://localhost:8083
-scidb_auth = ('foo', PASSWORD_PROVIDED)
-http_auth  = None
-namespace  = None
-verify     = False
+scidb_url         = https://localhost:8083
+scidb_auth        = ('foo', PASSWORD_PROVIDED)
+http_auth         = None
+verify            = False
+admin             = False
+namespace         = None
+use_arrow         = False
+result_size_limit = 256
+no_ops            = False
+
+
+When using the ``iquery`` SciDB client, the ``--admin`` flag is
+available for opening a higher-priority session. This flag is also
+available in SciDB-Py and can be set at connection time (see `SciDB
+Documentation <https://paradigm4.atlassian.net/wiki/spaces/scidb>`_
+for details on the effects of the flag). By default this flag is set
+to ``False``:
+
+>>> db_admin = connect(admin=True)
+
+>>> print(db_admin)
+scidb_url         = http://localhost:8080
+scidb_auth        = None
+http_auth         = None
+verify            = None
+admin             = True
+namespace         = None
+use_arrow         = False
+result_size_limit = 256
+no_ops            = False
 
 
 By default, the ``connect`` function queries SciDB for the list of
@@ -175,14 +229,23 @@ called on the ``DB`` instance:
 >>> db_no_ops.scan
 Traceback (most recent call last):
     ...
-AttributeError: 'DB' object has no attribute 'scan'
+AttributeError: Operators not loaded. Run 'load_ops()' or use \
+'no_ops=False' (default) at connection time (constructor)
 
 No query has been issued to SciDB yet.
 
 >>> db_no_ops.load_ops()
 >>> db_no_ops.scan
 ... # doctest: +NORMALIZE_WHITESPACE
-Operator(db=DB('http://localhost:8080', None, None, None, None),
+Operator(db=DB('http://localhost:8080',
+               None,
+               None,
+               None,
+               False,
+               None,
+               False,
+               256,
+               False),
          name='scan',
          args=[])
 
@@ -196,17 +259,37 @@ SciDB arrays can be accessed using ``DB.arrays``:
 >>> db.iquery('store(build(<x:int64>[i=0:2], i), foo)')
 
 >>> dir(db.arrays)
-... # doctest: +ELLIPSIS
-[...'foo']
+... # doctest: +ALLOW_UNICODE
+[u'foo']
 
 >>> dir(db.arrays.foo)
-... # doctest: +ELLIPSIS
-[...'i', ...'x']
+... # doctest: +ALLOW_UNICODE
+[u'i', u'x']
 
 >>> db.arrays.foo[:]
+   i    x
+0  0  0.0
+1  1  1.0
+2  2  2.0
+
+>>> db.arrays['foo'][:]
+   i    x
+0  0  0.0
+1  1  1.0
+2  2  2.0
+
+To get the schema of an array, we can use the ``schema`` utility
+function:
+
+>>> print(db.arrays.foo.schema())
+foo<x:int64> [i=0:2:0:1000000]
+
+>>> db.arrays.foo.schema().pprint()
 ... # doctest: +NORMALIZE_WHITESPACE
-array([(0, (255, 0)), (1, (255, 1)), (2, (255, 2))],
-      dtype=[('i', '<i8'), ('x', [('null', 'u1'), ('val', '<i8')])])
+foo<x:int64> [i=0:2:0:1000000]
+  name class   type nullable start end overlap    chunk
+0    x  attr  int64     True
+1    i   dim  int64              0   2       0  1000000
 
 >>> db.iquery('remove(foo)')
 
@@ -220,7 +303,7 @@ foo
 >>> print(db.arrays.bar)
 bar
 
-In IPython, you can use <TAB> for auto-completion of array names,
+In IPython, we can use <TAB> for auto-completion of array names,
 array dimensions, and array attributes::
 
     In [1]: db.arrays.<TAB>
@@ -237,15 +320,22 @@ operators and macros and makes them available through the ``DB`` class
 instance:
 
 >>> dir(db)
-... # doctest: +NORMALIZE_WHITESPACE +ELLIPSIS
-[...'aggregate',
- ...'apply',
+... # doctest: +NORMALIZE_WHITESPACE +ELLIPSIS +ALLOW_UNICODE
+[u'aggregate',
  ...
- ...'xgrid']
+ u'xgrid']
 
 >>> db.apply
 ... # doctest: +NORMALIZE_WHITESPACE
-Operator(db=DB('http://localhost:8080', None, None, None, None),
+Operator(db=DB('http://localhost:8080',
+               None,
+               None,
+               None,
+               False,
+               None,
+               False,
+               256,
+               False),
          name='apply',
          args=[])
 
@@ -257,20 +347,20 @@ Traceback (most recent call last):
     ...
 AttributeError: 'DB' object has no attribute 'missing'
 
-In IPython, you can use <TAB> for auto-completion of operator names::
+In IPython, we can use <TAB> for auto-completion of operator names::
 
     In [1]: db.<TAB>
     In [1]: db.apply
 
 The operators can be execute immediately or can be lazy and executed
 at a later time. Operators that return arrays are lazy operators (e.g.,
-``apply``, ``scan``, etc.). Operator which do not return arrays
+``apply``, ``scan``, etc.). Operators which do not return arrays
 execute immediately (e.g., ``create_array``, ``remove``, etc.).
 
 >>> db.create_array('foo', '<x:int64>[i]')
 >>> dir(db.arrays)
-... # doctest: +ELLIPSIS
-[...'foo']
+... # doctest: +ALLOW_UNICODE
+[u'foo']
 
 >>> db.remove(db.arrays.foo)
 >>> dir(db.arrays)
@@ -301,46 +391,86 @@ After SciDB restart:
 ... # doctest: +SKIP
 False
 
+The ``cross_join`` operator in SciDB supports aliasing for the array
+arguments, e.g., ``cross_join(left_array as left_alias,...``. Aliasing
+is possible in SciDB-Py using the ``%`` operator:
+
+>>> db.iquery('store(build(<x:int64>[i=0:1], i), foo)')
+>>> db.cross_join(db.arrays.foo % 'f1',
+...               db.arrays.foo % 'f2',
+...               'f1.i', 'f2.i')[:]
+   i    x  x_1
+0  0  0.0  0.0
+1  1  1.0  1.0
+
+>>> db.remove(db.arrays.foo)
+
+To retrieve the schema of a query result, the ``schema`` utility
+function can be used:
+
+>>> print(db.build('<x:int64>[i=0:2]', 'i').schema())
+build<x:int64> [i=0:2:0:1000000]
+
+>>> db.build('<x:int64>[i=0:2]', 'i').schema().pprint()
+... # doctest: +NORMALIZE_WHITESPACE
+build<x:int64> [i=0:2:0:1000000]
+  name class   type nullable start end overlap    chunk
+0    x  attr  int64     True
+1    i   dim  int64              0   2       0  1000000
+
+
 
 Download Data from SciDB
 ------------------------
 
 >>> db.build('<x:int8 not null>[i=0:2]', 'i + 10')[:]
-... # doctest: +NORMALIZE_WHITESPACE
-array([(0, 10), (1, 11), (2, 12)],
-      dtype=[('i', '<i8'), ('x', 'i1')])
-
->>> db.build('<x:int8 not null>[i=0:2]', 'i + 10').fetch(as_dataframe=True)
    i   x
 0  0  10
 1  1  11
 2  2  12
 
+>>> db.build('<x:int8 not null>[i=0:2]', 'i + 10').fetch(as_dataframe=False)
+... # doctest: +NORMALIZE_WHITESPACE
+array([(0, 10), (1, 11), (2, 12)],
+      dtype=[('i', '<i8'), ('x', 'i1')])
+
 >>> db.build('<x:int8 not null>[i=0:2]', 'i + 10').fetch(
-...     atts_only=True,
-...     as_dataframe=True)
+...     atts_only=True)
     x
 0  10
 1  11
 2  12
 
 >>> db.build('<x:int8 not null>[i=0:2]', 'i + 10').apply('y', 'x - 5')[:]
-... # doctest: +NORMALIZE_WHITESPACE
-array([(0, 10, 5), (1, 11, 6), (2, 12, 7)],
-      dtype=[('i', '<i8'), ('x', 'i1'), ('y', '<i8')])
+   i   x  y
+0  0  10  5
+1  1  11  6
+2  2  12  7
 
 >>> db.build('<x:int8 not null>[i=0:2]', 'i + 10').store('foo')
-Array(DB('http://localhost:8080', None, None, None, None), 'foo')
+... # doctest: +NORMALIZE_WHITESPACE
+Array(DB('http://localhost:8080',
+         None,
+         None,
+         None,
+         False,
+         None,
+         False,
+         256,
+         False),
+      'foo')
 
 >>> db.scan(db.arrays.foo)[:]
-... # doctest: +NORMALIZE_WHITESPACE
-array([(0, 10), (1, 11), (2, 12)],
-      dtype=[('i', '<i8'), ('x', 'i1')])
+   i   x
+0  0  10
+1  1  11
+2  2  12
 
 >>> db.apply(db.arrays.foo, 'y', db.arrays.foo.x + 1)[:]
-... # doctest: +NORMALIZE_WHITESPACE
-array([(0, 10, 11), (1, 11, 12), (2, 12, 13)],
-      dtype=[('i', '<i8'), ('x', 'i1'), ('y', '<i8')])
+   i   x   y
+0  0  10  11
+1  1  11  12
+2  2  12  13
 
 >>> db.remove(db.arrays.foo)
 
@@ -354,27 +484,37 @@ provided, it can be generated from the upload data or upload
 schema. If the upload format is not provided, it can be constructed
 from the upload schema, upload data, or resulting array schema.
 
-If an array name is not specified for the ``store`` operator, an array
-name is generated. Arrays with generated names are removed when the
-returned Array object is garbage collected. This behavior can be
-changed by specifying the ``gc=False`` argument to the store operator.
-
->>> ar = db.input(upload_data=numpy.arange(3)).store()
->>> ar
-... # doctest: +ELLIPSIS
-Array(DB('http://localhost:8080', None, None, None, None), 'py_...')
->>> del ar
-
 >>> db.input('<x:int64>[i]', upload_data=numpy.arange(3))[:]
-... # doctest: +NORMALIZE_WHITESPACE
-array([(0, (255, 0)), (1, (255, 1)), (2, (255, 2))],
-      dtype=[('i', '<i8'), ('x', [('null', 'u1'), ('val', '<i8')])])
+   i    x
+0  0  0.0
+1  1  1.0
+2  2  2.0
 
 >>> db.input('<x:int64>[i]', upload_data=numpy.arange(3)).store(db.arrays.foo)
-Array(DB('http://localhost:8080', None, None, None, None), 'foo')
+... # doctest: +NORMALIZE_WHITESPACE
+Array(DB('http://localhost:8080',
+         None,
+         None,
+         None,
+         False,
+         None,
+         False,
+         256,
+         False),
+      'foo')
 
 >>> db.load(db.arrays.foo, upload_data=numpy.arange(3))
-Array(DB('http://localhost:8080', None, None, None, None), 'foo')
+... # doctest: +NORMALIZE_WHITESPACE
+Array(DB('http://localhost:8080',
+         None,
+         None,
+         None,
+         False,
+         None,
+         False,
+         256,
+         False),
+      'foo')
 
 >>> db.input('<x:int64>[j]', upload_data=numpy.arange(3, 6)
 ...  ).apply('i', 'j + 3'
@@ -382,28 +522,63 @@ Array(DB('http://localhost:8080', None, None, None, None), 'foo')
 ...  ).insert(db.arrays.foo)
 
 >>> db.arrays.foo[:]
-... # doctest: +NORMALIZE_WHITESPACE
-array([(0, (255, 0)), (1, (255, 1)), (2, (255, 2)), (3, (255, 3)),
-       (4, (255, 4)), (5, (255, 5))],
-      dtype=[('i', '<i8'), ('x', [('null', 'u1'), ('val', '<i8')])])
+   i    x
+0  0  0.0
+1  1  1.0
+2  2  2.0
+3  3  3.0
+4  4  4.0
+5  5  5.0
 
->>> db.input('<i:int64 not null, x:int64>[j]', upload_data=db.arrays.foo[:]
+>>> db.input('<i:int64 not null, x:int64>[j]',
+...          upload_data=db.arrays.foo.fetch(as_dataframe=False)
 ...  ).redimension(db.arrays.foo
 ...  ).store('bar')
-Array(DB('http://localhost:8080', None, None, None, None), 'bar')
+... # doctest: +NORMALIZE_WHITESPACE
+Array(DB('http://localhost:8080',
+         None,
+         None,
+         None,
+         False,
+         None,
+         False,
+         256,
+         False),
+      'bar')
 
->>> numpy.all(db.arrays.bar[:] == db.arrays.foo[:])
+>>> numpy.all(db.arrays.bar.fetch(as_dataframe=False)
+...           == db.arrays.foo.fetch(as_dataframe=False))
 True
 
 >>> buf = numpy.array([bytes([10, 20, 30])], dtype='object')
 
 >>> db.input('<b:binary not null>[i]', upload_data=buf).store('taz')
-Array(DB('http://localhost:8080', None, None, None, None), 'taz')
+... # doctest: +NORMALIZE_WHITESPACE
+Array(DB('http://localhost:8080',
+         None,
+         None,
+         None,
+         False,
+         None,
+         False,
+         256,
+         False),
+      'taz')
 
 >>> db.load('taz',
 ...         upload_data=buf,
 ...         upload_schema=Schema.fromstring('<b:binary not null>[i]'))
-Array(DB('http://localhost:8080', None, None, None, None), 'taz')
+... # doctest: +NORMALIZE_WHITESPACE
+Array(DB('http://localhost:8080',
+         None,
+         None,
+         None,
+         False,
+         None,
+         False,
+         256,
+         False),
+      'taz')
 
 For files already available on the server the ``input`` or ``load``
 operators can be invoked with the full set of arguments supported by
@@ -415,6 +590,36 @@ double-quoted in SciDB-Py. For example:
 
 >>> for ar in ['foo', 'bar', 'taz']: db.remove(ar)
 
+The ``store`` function accepts a ``temp`` argument as well. If the
+``temp`` argument is set to ``True``, a temporary array is created.
+
+>>> ar = db.build('<x:int64>[i=0:2]', 'i').store('foo', temp=True)
+>>> db.list()[['name', 'temporary']]
+  name  temporary
+0  foo       True
+>>> db.remove(ar)
+
+If an array name is not specified for the ``store`` operator, an array
+name is generated. Arrays with generated names are removed when the
+returned Array object is garbage collected. This behavior can be
+changed by specifying the ``gc=False`` argument to the store operator.
+
+>>> ar = db.input(upload_data=numpy.arange(3)).store()
+>>> ar
+... # doctest: +ELLIPSIS +NORMALIZE_WHITESPACE
+Array(DB('http://localhost:8080',
+         None,
+         None,
+         None,
+         False,
+         None,
+         False,
+         256,
+         False),
+      'py_...')
+>>> del ar
+
+
 The iquery Function
 ===================
 
@@ -431,17 +636,19 @@ The ``iquery`` function can be used to download data from SciDB by
 specifying the ``fetch=True`` argument:
 
 >>> db.iquery('scan(foo)', fetch=True)
-... # doctest: +NORMALIZE_WHITESPACE
-array([(0, (255, 0)), (1, (255, 1)), (2, (255, 2))],
-      dtype=[('i', '<i8'), ('x', [('null', 'u1'), ('val', '<i8')])])
+   i    x
+0  0  0.0
+1  1  1.0
+2  2  2.0
 
 To avoid downloading the dimension information and only download the
 attributes, use the ``atts_only=True`` argument:
 
 >>> db.iquery('scan(foo)', fetch=True, atts_only=True)
-... # doctest: +NORMALIZE_WHITESPACE
-array([((255, 0),), ((255, 1),), ((255, 2),)],
-      dtype=[('x', [('null', 'u1'), ('val', '<i8')])])
+     x
+0  0.0
+1  1.0
+2  2.0
 
 >>> db.iquery('remove(foo)')
 
@@ -450,25 +657,28 @@ Download operator output directly:
 
 >>> db.iquery('build(<x:int64 not null>[i=0:2], i)',
 ...           fetch=True)
-... # doctest: +NORMALIZE_WHITESPACE
-array([(0, 0), (1, 1), (2, 2)],
-      dtype=[('i', '<i8'), ('x', '<i8')])
+   i  x
+0  0  0
+1  1  1
+2  2  2
 
 >>> db.iquery('build(<x:int64 not null>[i=0:2], i)',
 ...           fetch=True,
 ...           atts_only=True)
-... # doctest: +NORMALIZE_WHITESPACE
-array([(0,), (1,), (2,)],
-      dtype=[('x', '<i8')])
+   x
+0  0
+1  1
+2  2
 
 
 If dimension names collide with attribute names, unique dimension
 names are created:
 
 >>> db.iquery('apply(build(<x:int64 not null>[i=0:2], i), i, i)', fetch=True)
-... # doctest: +NORMALIZE_WHITESPACE
-array([(0, 0, 0), (1, 1, 1), (2, 2, 2)],
-      dtype=[('i_1', '<i8'), ('x', '<i8'), ('i', '<i8')])
+   i_1  x  i
+0    0  0  0
+1    1  1  1
+2    2  2  2
 
 
 If schema is known, it can be provided to ``iquery`` using the
@@ -483,42 +693,35 @@ schema:
 ...        schema=Schema(None,
 ...                      (Attribute('x', 'int64', not_null=True),),
 ...                      (Dimension('i', 0, 2),)))
-... # doctest: +NORMALIZE_WHITESPACE
-array([(0, 0), (1, 1), (2, 2)],
-      dtype=[('i', '<i8'), ('x', '<i8')])
+   i  x
+0  0  0
+1  1  1
+2  2  2
 
 >>> iquery(db,
 ...        'build(<x:int64 not null>[i=0:2], i)',
 ...        fetch=True,
 ...        atts_only=True,
 ...        schema=Schema.fromstring('<x:int64 not null>[i=0:2]'))
-... # doctest: +NORMALIZE_WHITESPACE
-array([(0,), (1,), (2,)],
-      dtype=[('x', '<i8')])
-
-
-Download as Pandas DataFrame:
-
->>> iquery(db,
-...        'build(<x:int64>[i=0:2], i)',
-...        fetch=True,
-...        as_dataframe=True)
-... # doctest: +NORMALIZE_WHITESPACE
-   i x
-0  0 0.0
-1  1 1.0
-2  2 2.0
-
->>> iquery(db,
-...        'build(<x:int64>[i=0:2], i)',
-...        fetch=True,
-...        atts_only=True,
-...        as_dataframe=True)
 ... # doctest: +NORMALIZE_WHITESPACE
    x
-0  0.0
-1  1.0
-2  2.0
+0  0
+1  1
+2  2
+
+
+Attributes with null-able types are promoted as per Pandas `promotion
+scheme <http://pandas.pydata.org/pandas-docs/stable/gotchas.html
+#na-type-promotions>`_ During such promotions, precision loss might
+occur. The user is notified about this via a ``UserWarning``. If
+desired, such warnings can be disabled as follows:
+
+>>> import warnings
+>>> warnings.filterwarnings('ignore', category = UserWarning)
+
+Type promotions can be avoided altogether, by specifying
+``dataframe_promo = False`` in the ``iquery`` arguments. In this case,
+object records will be used instead of atomic values:
 
 >>> iquery(db,
 ...        'build(<x:int64>[i=0:2], i)',
@@ -533,6 +736,53 @@ Download as Pandas DataFrame:
 2  (255, 2)
 
 
+Download as NumPy Array:
+
+>>> iquery(db,
+...        'build(<x:int64>[i=0:2], i)',
+...        fetch=True,
+...        as_dataframe=False)
+... # doctest: +NORMALIZE_WHITESPACE
+array([(0, (255, 0)), (1, (255, 1)), (2, (255, 2))],
+      dtype=[('i', '<i8'), ('x', [('null', 'u1'), ('val', '<i8')])])
+
+>>> iquery(db,
+...        'build(<x:int64>[i=0:2], i)',
+...        fetch=True,
+...        atts_only=True,
+...        as_dataframe=False)
+... # doctest: +NORMALIZE_WHITESPACE
+array([((255, 0),), ((255, 1),), ((255, 2),)],
+      dtype=[('x', [('null', 'u1'), ('val', '<i8')])])
+
+
+If the `accelerated_io_tools
+<https://github.com/Paradigm4/accelerated_io_tools>`_ SciDB plugin is
+installed and enabled in `Shim <https://github.com/Paradigm4/shim>`_,
+SciDB arrays can be downloaded using the Apache Arrow library:
+
+>>> db.iquery('scan(foo)', fetch=True, use_arrow=True)
+... # doctest: +SKIP
+   i    x
+0  0  0.0
+1  1  1.0
+2  2  2.0
+
+
+By default, Apache Arrow is not used. If desired, ``use_arrow`` can be
+set to ``True`` at connection time (it is ``False`` by default). Once
+set at connection time, this value is used for any subsequent
+``iquery`` calls if not overwritten:
+
+>>> db_arrow = connect(use_arrow=True)
+>>> db_arrow.iquery('scan(foo)', fetch=True)
+... # doctest: +SKIP
+   i    x
+0  0  0.0
+1  1  1.0
+2  2  2.0
+
+
 Upload Data to SciDB
 --------------------
 
@@ -558,6 +808,7 @@ The SciDB query placeholders are:
 
 See examples in the following subsections.
 
+
 Upload NumPy Arrays
 ^^^^^^^^^^^^^^^^^^^
 
@@ -570,9 +821,10 @@ populate these placeholders.
 ...           upload_data=numpy.arange(3))
 
 >>> db.arrays.foo[:]
-... # doctest: +NORMALIZE_WHITESPACE
-array([(0, (255, 0)), (1, (255, 1)), (2, (255, 2))],
-      dtype=[('i', '<i8'), ('x', [('null', 'u1'), ('val', '<i8')])])
+   i    x
+0  0  0.0
+1  1  1.0
+2  2  2.0
 
 >>> db.iquery("insert(input({sch}, '{fn}', 0, '(int64)'), foo)",
 ...           upload_data=numpy.arange(3))
@@ -637,6 +889,77 @@ library. The data file object is passed directly to the ``requests``
 library which handles the HTTP communication with *Shim*.
 
 
+Joins
+=====
+
+SciDB provides a number of join operators, including ``join``,
+``cross_join``, etc. As with any other operators, these operators can
+be used as literals in the ``DB.iquery`` function or as functions of
+the ``DB`` object. Here are some join examples exposing various
+features of the library:
+
+>>> foo = db.build('<val:double>[i=0:2; j=0:2]', 'i * 3 + j').store('foo')
+
+>>> db.join(foo,
+...         db.build('<val:double>[i=0:2; j=0:5]', '0'))[:]
+   i  j  val  val_1
+0  0  0  0.0    0.0
+1  0  1  1.0    0.0
+2  0  2  2.0    0.0
+3  1  0  3.0    0.0
+4  1  1  4.0    0.0
+5  1  2  5.0    0.0
+6  2  0  6.0    0.0
+7  2  1  7.0    0.0
+8  2  2  8.0    0.0
+
+>>> db.build('<val:double>[i=0:2; j=0:5]', '0'
+...  ).join(db.arrays.foo)[:]
+   i  j  val  val_1
+0  0  0  0.0    0.0
+1  0  1  0.0    1.0
+2  0  2  0.0    2.0
+3  1  0  0.0    3.0
+4  1  1  0.0    4.0
+5  1  2  0.0    5.0
+6  2  0  0.0    6.0
+7  2  1  0.0    7.0
+8  2  2  0.0    8.0
+
+>>> db.cross_join(foo,
+...               db.build('<val:double>[k=0:5]', 'k + 100'),
+...               foo.j, 'k')[:]
+   i  j  val  val_1
+0  0  0  0.0  100.0
+1  0  1  1.0  101.0
+2  0  2  2.0  102.0
+3  1  0  3.0  100.0
+4  1  1  4.0  101.0
+5  1  2  5.0  102.0
+6  2  0  6.0  100.0
+7  2  1  7.0  101.0
+8  2  2  8.0  102.0
+
+>>> bar = db.build('<val:double>[j=0:5]', 'j + 100').store()
+
+>>> db.cross_join(db.arrays.foo % 'left',
+...               bar % 'right',
+...               'left.j', 'right.j')[:]
+   i  j  val  val_1
+0  0  0  0.0  100.0
+1  0  1  1.0  101.0
+2  0  2  2.0  102.0
+3  1  0  3.0  100.0
+4  1  1  4.0  101.0
+5  1  2  5.0  102.0
+6  2  0  6.0  100.0
+7  2  1  7.0  101.0
+8  2  2  8.0  102.0
+
+>>> db.remove(foo)
+>>> db.remove(bar)
+
+
 SciDB Enterprise Edition Features
 =================================
 
@@ -649,11 +972,15 @@ function. No immediate query is executed in SciDB, but the new
 namespace will take effect for any subsequent SciDB queries:
 
 >>> print(db)
-scidb_url  = http://localhost:8080
-scidb_auth = None
-http_auth  = None
-namespace  = None
-verify     = None
+scidb_url         = http://localhost:8080
+scidb_auth        = None
+http_auth         = None
+verify            = None
+admin             = False
+namespace         = None
+use_arrow         = False
+result_size_limit = 256
+no_ops            = False
 
 Notice the ``namespace`` field of the ``DB`` instance.
 
@@ -661,22 +988,30 @@ Notice the ``namespace`` field of the ``DB`` instance.
 ... # doctest: +SKIP
 >>> print(db)
 ... # doctest: +SKIP
-scidb_url  = http://localhost:8080
-scidb_auth = None
-http_auth  = None
-namespace  = private
-verify     = None
+scidb_url         = http://localhost:8080
+scidb_auth        = None
+http_auth         = None
+verify            = None
+admin             = False
+namespace         = private
+use_arrow         = False
+result_size_limit = 256
+no_ops            = False
 >>> db.show_namespace()[0]['name']['val']
 ... # doctest: +SKIP
 'private'
 
 >>> db.iquery("set_namespace('public')")
 >>> print(db)
-scidb_url  = http://localhost:8080
-scidb_auth = None
-http_auth  = None
-namespace  = public
-verify     = None
+scidb_url         = http://localhost:8080
+scidb_auth        = None
+http_auth         = None
+verify            = None
+admin             = False
+namespace         = public
+use_arrow         = False
+result_size_limit = 256
+no_ops            = False
 >>> db.show_namespace()[0]['name']['val']
 ... # doctest: +SKIP
 'public'
@@ -690,15 +1025,19 @@ time:
 ... # doctest: +SKIP
 >>> print(db_ssl)
 ... # doctest: +SKIP
-scidb_url  = https://localhost:8083
-scidb_auth = None
-http_auth  = None
-namespace  = public
-verify     = False
+scidb_url         = https://localhost:8083
+scidb_auth        = None
+http_auth         = None
+verify            = False
+admin             = Flase
+namespace         = public
+use_arrow         = False
+result_size_limit = 256
+no_ops            = False
 
 """
 
 from .db import connect, iquery, Array
 from .schema import Attribute, Dimension, Schema
 
-__version__ = '16.9.1'
+__version__ = '18.1.5'
