@@ -12,10 +12,12 @@ import logging
 import numpy
 import os
 import pandas
+import pyarrow
 import re
 import requests
 import string
 import threading
+import uuid
 import warnings
 
 try:
@@ -32,6 +34,7 @@ class Shim(enum.Enum):
     execute_query = 'execute_query'
     new_session = 'new_session'
     read_bytes = 'read_bytes'
+    read_lines = 'read_lines'
     release_session = 'release_session'
     upload = 'upload'
 
@@ -41,53 +44,98 @@ class Password_Placeholder(object):
         return 'PASSWORD_PROVIDED'
 
 
+def _shim_release_session(scidb_url, http_auth, verify, id):
+    """Make Shim release_session request"""
+    url = requests.compat.urljoin(scidb_url, Shim.release_session.value)
+    req = requests.get(
+        url,
+        params={'id': id},
+        auth=http_auth,
+        verify=verify)
+    req.reason = req.content
+    req.raise_for_status()
+
+
 class DB(object):
     """SciDB Shim connection object.
 
     >>> DB()
-    DB('http://localhost:8080', None, None, None, None)
+    ... # doctest: +NORMALIZE_WHITESPACE
+    DB('http://localhost:8080',
+       None,
+       None,
+       None,
+       False,
+       None,
+       False,
+       256,
+       False)
 
     >>> print(DB())
-    scidb_url  = http://localhost:8080
-    scidb_auth = None
-    http_auth  = None
-    namespace  = None
-    verify     = None
+    scidb_url         = http://localhost:8080
+    scidb_auth        = None
+    http_auth         = None
+    verify            = None
+    admin             = False
+    namespace         = None
+    use_arrow         = False
+    result_size_limit = 256
+    no_ops            = False
 
     Constructor parameters:
 
     :param string scidb_url: SciDB connection URL. The URL for the
-      Shim server. If `None`, use the value of the `SCIDB_URL`
+      Shim server. If ``None``, use the value of the ``SCIDB_URL``
       environment variable, if present (default
-      `http://localhost:8080`)
+      ``http://localhost:8080``)
 
     :param tuple scidb_auth: Tuple with username and password for
       connecting to SciDB, if password authentication method is used
-      (default `None`)
+      (default ``None``)
 
     :param tuple http_auth: Tuple with username and password for
       connecting to Shim, if Shim authentication is used (default
-      `None`)
+      ``None``)
 
-    :param string namespace: Initial namespace for the
-      connection. Only applicable for SciDB Enterprise Edition. The
-      namespace can changed at any time using the `set_namespace`
-      SciDB operator (default `None`)
-
-    :param bool verify: If `False`, HTTPS certificates are not
-      verified. This value is passed to the Python `requests`
+    :param bool verify: If ``False``, HTTPS certificates are not
+      verified. This value is passed to the Python ``requests``
       library. See Python `requests
       <http://docs.python-requests.org/en/master/>`_ library `SSL Cert
       Verification
       <http://docs.python-requests.org/en/master/user/advanced/
       #ssl-cert-verification>`_ section for details on the ``verify``
-      argument (default `None`)
+      argument (default ``None``)
 
-    :param bool no_ops: If `True`, the list of operators is not
+    :param bool admin: Set to ``True`` to open a higher-priority
+      session. This is identical with the ``--admin`` flag for the
+      ``iquery`` SciDB client, see `SciDB Documentation
+      <https://paradigm4.atlassian.net/wiki/spaces/scidb>`_ for
+      details (default ``False``)
+
+    :param string namespace: Initial namespace for the
+      connection. Only applicable for SciDB Enterprise Edition. The
+      namespace can changed at any time using the ``set_namespace``
+      SciDB operator (default ``None``)
+
+    :param bool use_arrow: If ``True``, download SciDB array using
+      Apache Arrow library. Requires ``accelerated_io_tools`` and
+      ``aio`` enabled in ``Shim``. If ``True``, a Pandas DataFrame is
+      returned (``as_dataframe`` has no effect) and null-able types
+      are promoted as per Pandas `promotion scheme
+      <http://pandas.pydata.org/pandas-docs/stable/gotchas.html
+      #na-type-promotions>`_ (``dataframe_promo`` has no effect). It
+      can be overriden for each ``iquery`` call (default ``False``)
+
+    :param int result_size_limit: absolute limit of the output file in
+      Megabytes. Effective only when the `accelerated_io_tools`
+      plug-in is installed in SciDB and `aio` is enabled in Shim
+      (default `256` MB)
+
+    :param bool no_ops: If ``True``, the list of operators is not
       fetched at this time and the connection is not implicitly
       verified. This expedites the execution of the function but
-      disallows for calling the SciDB operators directly from the `DB`
-      instance e.g., `db.scan` (default `False`)
+      disallows for calling the SciDB operators directly from the
+      ``DB`` instance e.g., ``db.scan`` (default ``False``)
 
     """
 
@@ -98,15 +146,22 @@ class DB(object):
             scidb_url=None,
             scidb_auth=None,
             http_auth=None,
-            namespace=None,
             verify=None,
+            admin=False,
+            namespace=None,
+            use_arrow=False,
+            result_size_limit=256,
             no_ops=False):
         if scidb_url is None:
             scidb_url = os.getenv('SCIDB_URL', 'http://localhost:8080')
 
         self.scidb_url = scidb_url
-        self.namespace = namespace
         self.verify = verify
+        self.admin = admin
+        self.namespace = namespace
+        self.use_arrow = use_arrow
+        self.result_size_limit = result_size_limit
+        self.no_ops = no_ops
 
         if http_auth:
             self._http_auth = requests.auth.HTTPDigestAuth(*http_auth)
@@ -114,26 +169,33 @@ class DB(object):
         else:
             self._http_auth = self.http_auth = None
 
+        admin_shim = 1 if self.admin else 0
         if scidb_auth:
-            if not self.scidb_url.lower().startswith('https'):
-                raise Exception(
-                    'SciDB credentials can only be used ' +
-                    'with https connections')
-
-            self._scidb_auth = {'user': scidb_auth[0],
-                                'password': scidb_auth[1]}
+            self._id = self._shim(Shim.new_session,
+                                  user=scidb_auth[0],
+                                  password=scidb_auth[1],
+                                  admin=admin_shim).text
             self.scidb_auth = (scidb_auth[0], Password_Placeholder())
         else:
-            self._scidb_auth = self.scidb_auth = None
+            self._id = self._shim(Shim.new_session,
+                                  admin=admin_shim).text
+            self.scidb_auth = None
+
+        finalize(self,
+                 _shim_release_session,
+                 self.scidb_url,
+                 self._http_auth,
+                 self.verify,
+                 self._id)
 
         self.arrays = Arrays(self)
 
-        self._id = None
+        self._uid = uuid.uuid1().hex
         self._lock = threading.Lock()
         self._array_cnt = 0
         self._formatter = string.Formatter()
 
-        if no_ops:
+        if self.no_ops:
             self.operators = None
             self._dir = None
         else:
@@ -144,24 +206,46 @@ class DB(object):
             self.scidb_url,
             self.scidb_auth,
             self.http_auth,
+            self.verify,
+            self.admin,
             self.namespace,
-            self.verify))
+            self.use_arrow,
+            self.result_size_limit,
+            self.no_ops))
 
     def __repr__(self):
-        return '{}({!r}, {!r}, {!r}, {!r}, {!r})'.format(
+        return ('{}({!r}, ' +
+                '{!r}, ' +
+                '{!r}, ' +
+                '{!r}, ' +
+                '{!r}, ' +
+                '{!r}, ' +
+                '{!r}, ' +
+                '{!r}, ' +
+                '{!r})').format(
             type(self).__name__, *self)
 
     def __str__(self):
         return '''\
-scidb_url  = {}
-scidb_auth = {}
-http_auth  = {}
-namespace  = {}
-verify     = {}'''.format(*self)
+scidb_url         = {}
+scidb_auth        = {}
+http_auth         = {}
+verify            = {}
+admin             = {}
+namespace         = {}
+use_arrow         = {}
+result_size_limit = {}
+no_ops            = {}'''.format(*self)
 
     def __getattr__(self, name):
         if self.operators and name in self.operators:
             return Operator(self, name)
+        elif self.no_ops:
+            raise AttributeError(
+                ("Operators not loaded. Run 'load_ops()' or " +
+                 "use 'no_ops=False' (default) " +
+                 "at connection time (constructor)").format(
+                     type(self)))
         else:
             raise AttributeError(
                 '{.__name__!r} object has no attribute {!r}'.format(
@@ -173,8 +257,9 @@ verify     = {}'''.format(*self)
     def iquery(self,
                query,
                fetch=False,
+               use_arrow=None,
                atts_only=False,
-               as_dataframe=False,
+               as_dataframe=True,
                dataframe_promo=True,
                schema=None,
                upload_data=None,
@@ -183,45 +268,58 @@ verify     = {}'''.format(*self)
 
         :param string query: SciDB AFL query to execute
 
-        :param bool fetch: If `True`, download SciDB array (default
-          `False`)
+        :param bool fetch: If ``True``, download SciDB array (default
+          ``False``)
 
-        :param bool atts_only: If `True`, download only SciDB array
-          attributes without dimensions (default `False`)
-
-        :param bool as_dataframe: If `True`, return a Pandas
-          DataFrame. If `False`, return a NumPy array (default
-          `False`)
-
-        :param bool dataframe_promo: If `True`, null-able types are
-          promoted as per Pandas 'promotion scheme
+        :param bool use_arrow: If ``True``, download SciDB array using
+          Apache Arrow library. Requires ``accelerated_io_tools`` and
+          ``aio`` enabled in ``Shim``. If ``True``, a Pandas DataFrame
+          is returned (``as_dataframe`` has no effect) and null-able
+          types are promoted as per Pandas `promotion scheme
           <http://pandas.pydata.org/pandas-docs/stable/gotchas.html
-          #na-type-promotions>`_ If `False`, object records are used
-          for null-able types (default `True`)
+          #na-type-promotions>`_ (``dataframe_promo`` has no
+          effect). If ``None`` the ``use_arrow`` value set at
+          connection time is used (default ``None``)
+
+        :param bool atts_only: If ``True``, download only SciDB array
+          attributes without dimensions (default ``False``)
+
+        :param bool as_dataframe: If ``True``, return a Pandas
+          DataFrame. If ``False``, return a NumPy array (default
+          ``True``)
+
+        :param bool dataframe_promo: If ``True``, null-able types are
+          promoted as per Pandas `promotion scheme
+          <http://pandas.pydata.org/pandas-docs/stable/gotchas.html
+          #na-type-promotions>`_ If ``False``, object records are used
+          for null-able types (default ``True``)
 
         :param schema: Schema of the SciDB array to use when
           downloading the array. Schema is not verified. If schema is
           a Schema instance, it is copied. Otherwise, a
-          :py:class:`Schema` object is built using
-          :py:func:`Schema.fromstring` (default `None`)
+          :py:class:``Schema`` object is built using
+          :py:func:``Schema.fromstring`` (default ``None``)
 
         >>> DB().iquery('build(<x:int64>[i=0:1; j=0:1], i + j)', fetch=True)
-        ... # doctest: +NORMALIZE_WHITESPACE
-        array([(0, 0, (255, 0)),
-               (0, 1, (255, 1)),
-               (1, 0, (255, 1)),
-               (1, 1, (255, 2))],
-              dtype=[('i', '<i8'), ('j', '<i8'),
-                     ('x', [('null', 'u1'), ('val', '<i8')])])
+           i  j    x
+        0  0  0  0.0
+        1  0  1  1.0
+        2  1  0  1.0
+        3  1  1  2.0
 
         >>> DB().iquery("input({sch}, '{fn}', 0, '{fmt}')",
         ...             fetch=True,
         ...             upload_data=numpy.arange(3, 6))
-        ... # doctest: +NORMALIZE_WHITESPACE
-        array([(0, 3), (1, 4), (2, 5)],
-              dtype=[('i', '<i8'), ('x', '<i8')])
+           i  x
+        0  0  3
+        1  1  4
+        2  2  5
 
         """
+        # Set use_arrow using local/global
+        if use_arrow is None:
+            use_arrow = self.use_arrow
+
         # Special case: -- - set_namespace - --
         if query.startswith('set_namespace(') and query[-1] == ')':
             param = query[len('set_namespace('):-1]
@@ -231,8 +329,6 @@ verify     = {}'''.format(*self)
             self.namespace = param
             return
 
-        id = self._shim(Shim.new_session).text
-
         if upload_data is not None:
             if isinstance(upload_data, numpy.ndarray):
                 if upload_schema is None:
@@ -268,12 +364,11 @@ verify     = {}'''.format(*self)
             if not (isinstance(upload_data, bytes) or
                     isinstance(upload_data, bytearray) or
                     hasattr(upload_data, 'read')):
-                print('data type')
                 warnings.warn(
                     'upload_data is not bytes or file-like object',
                     stacklevel=2)
 
-            fn = self._shim(Shim.upload, id=id, data=upload_data).text
+            fn = self._shim(Shim.upload, data=upload_data).text
             query = query.format(
                 sch=upload_schema,
                 fn=fn,
@@ -284,7 +379,7 @@ verify     = {}'''.format(*self)
             if schema:
                 # Deep-copy schema since we might be mutating it
                 if isinstance(schema, Schema):
-                    if not atts_only:
+                    if not atts_only and not use_arrow:
                         schema = copy.deepcopy(schema)
                 else:
                     schema = Schema.fromstring(schema)
@@ -292,11 +387,10 @@ verify     = {}'''.format(*self)
                 # Execute 'show(...)' and Download text
                 self._shim(
                     Shim.execute_query,
-                    id=id,
                     query=DB._show_query.format(query.replace("'", "\\'")),
                     save='tsv')
                 schema = Schema.fromstring(
-                    self._shim(Shim.read_bytes, id=id, n=0).text)
+                    self._shim(Shim.read_lines, n=0).text)
 
             # Attributes and dimensions can collide. Run make_unique to
             # remove any collisions.
@@ -314,7 +408,7 @@ verify     = {}'''.format(*self)
                 query = 'cast({}, {:h})'.format(query, schema)
 
             # Unpack
-            if not atts_only:
+            if not atts_only and not use_arrow:
                 # apply: add dimensions as attributes
                 # project: place dimensions first
                 query = 'project(apply({}, {}), {})'.format(
@@ -328,27 +422,40 @@ verify     = {}'''.format(*self)
 
             # Execute Query and Download content
             self._shim(Shim.execute_query,
-                       id=id,
                        query=query,
-                       save=schema.atts_fmt_scidb)
-            buf = self._shim(Shim.read_bytes, id=id, n=0).content
+                       save='arrow' if use_arrow else schema.atts_fmt_scidb,
+                       result_size_limit=self.result_size_limit,
+                       atts_only=1 if atts_only or not use_arrow else 0)
+            buf = self._shim(Shim.read_bytes, n=0).content
 
-            self._shim(Shim.release_session, id=id)
+            # Build result
+            if use_arrow:
+                data = pyarrow.RecordBatchStreamReader(
+                    pyarrow.BufferReader(buf)).read_pandas()
+                # Place dimensions first
+                if not atts_only:
+                    data = data[[i.name for i in itertools.chain(
+                        schema.dims, schema.atts)]]
 
-            if schema.is_fixsize() and (not as_dataframe or
-                                        not dataframe_promo):
+            elif schema.is_fixsize():
                 data = numpy.frombuffer(buf, dtype=schema.atts_dtype)
+
+                if as_dataframe:
+                    data = pandas.DataFrame.from_records(data)
+
+                    if dataframe_promo:
+                        schema.promote(data)
             else:
+                # Parse binary buffer
                 data = schema.frombytes(buf, as_dataframe, dataframe_promo)
 
-            # Return NumPy array or Pandas dataframe
-            if as_dataframe:
-                return pandas.DataFrame.from_records(data)
-            else:
-                return data
+                if as_dataframe:
+                    data = pandas.DataFrame.from_records(data)
+
+            return data
 
         else:                   # fetch=False
-            self._shim(Shim.execute_query, id=id, query=query, release=1)
+            self._shim(Shim.execute_query, query=query)
 
             # Special case: -- - load_library - --
             if query.startswith('load_library('):
@@ -366,45 +473,35 @@ verify     = {}'''.format(*self)
         ... # doctest: +ELLIPSIS
         [[...'0', ...'10'], [...'1', ...'11'], [...'2', ...'12']]
         """
-        id = self._shim(Shim.new_session).text
-        self._shim(Shim.execute_query, id=id, query=query, save='tsv')
-        ret = self._shim_readlines(id=id)
-        self._shim(Shim.release_session, id=id)
+        self._shim(Shim.execute_query, query=query, save='tsv')
+        ret = self._shim_readlines()
         return ret
 
     def next_array_name(self):
         """Generate a uniqu array name. Keep track on these names using the
-           _id field and a conter
+           _uid field and a counter
         """
         # Thread-safe counter
         with self._lock:
             self._array_cnt += 1
-            return 'py_{}_{}'.format(self._id, self._array_cnt)
+            return 'py_{}_{}'.format(self._uid, self._array_cnt)
 
     def load_ops(self):
-        """Get list of operators and macros. Also sets the _id field used to
-           generate unique array names
+        """Get list of operators and macros.
         """
-        id = self._shim(Shim.new_session).text
-
         query_id = self._shim(
             Shim.execute_query,
-            id=id,
             query="project(list('operators'), name)",
             save='tsv').text  # set query ID as DB instance ID
-        if self._id is None:
-            self._id = query_id
-        operators = self._shim_readlines(id=id)
+        operators = self._shim_readlines()
 
         self._shim(
             Shim.execute_query,
-            id=id,
             query="project(list('macros'), name)",
             save='tsv').content
-        macros = self._shim_readlines(id=id)
-
-        self._shim(Shim.release_session, id=id)
+        macros = self._shim_readlines()
 
+        self.no_ops = False
         self.operators = operators + macros
         self._dir = (self.operators +
                      ['arrays',
@@ -417,9 +514,11 @@ verify     = {}'''.format(*self)
     def _shim(self, endpoint, **kwargs):
         """Make request on Shim endpoint"""
 
-        # Add credentails to request, if necessary
-        if self._scidb_auth and endpoint in (Shim.cancel, Shim.execute_query):
-            kwargs.update(self._scidb_auth)
+        if endpoint != Shim.new_session:
+            kwargs.update(id=self._id)
+
+        if self.use_arrow and endpoint == Shim.execute_query:
+            kwargs['result_size_limit'] = self.result_size_limit
 
         # Add prefix to request, if necessary
         if self.namespace and endpoint == Shim.execute_query:
@@ -434,20 +533,31 @@ verify     = {}'''.format(*self)
                 auth=self._http_auth,
                 verify=self.verify)
         else:                        # Get request
-            req = requests.get(
-                url,
-                params=kwargs,
-                auth=self._http_auth,
-                verify=self.verify)
+            try:
+                req = requests.get(
+                    url,
+                    params=kwargs,
+                    auth=self._http_auth,
+                    verify=self.verify)
+            except KeyboardInterrupt as e:
+                if endpoint == Shim.execute_query:
+                    requests.get(
+                        requests.compat.urljoin(
+                            self.scidb_url, Shim.cancel.value),
+                        params={'id': self._id},
+                        auth=self._http_auth,
+                        verify=self.verify)
+                raise e
+
         req.reason = req.content
         req.raise_for_status()
         return req
 
-    def _shim_readlines(self, id):
+    def _shim_readlines(self):
         """Read data from Shim and parse as text lines"""
         return [line.split('\t') if '\t' in line else line
                 for line in self._shim(
-                        Shim.read_bytes, id=id, n=0).text.splitlines()]
+                        Shim.read_lines, n=0).text.splitlines()]
 
 
 class Arrays(object):
@@ -464,6 +574,11 @@ class Arrays(object):
 {}'''.format(self.db)
 
     def __getattr__(self, name):
+        """db.arrays.foo"""
+        return Array(self.db, name)
+
+    def __getitem__(self, name):
+        """db.arrays['foo']"""
         return Array(self.db, name)
 
     def __dir__(self):
@@ -498,16 +613,41 @@ class Array(object):
         return self.fetch()[key]
 
     def __dir__(self):
-        """Download the schema of the SciDB array, using `show()`"""
+        """Download the schema of the SciDB array, using ``show()``"""
         sh = Schema.fromstring(
             self.db.iquery_readlines('show({})'.format(self))[0])
         ls = [i.name for i in itertools.chain(sh.atts, sh.dims)]
         ls.sort()
         return ls
 
-    def fetch(self, as_dataframe=False):
-        return self.db.iquery(
-            'scan({})'.format(self), fetch=True, as_dataframe=as_dataframe)
+    def __mod__(self, alias):
+        """Overloads ``%`` operator to add support for aliasing"""
+        return Array(self.db, '{} as {}'.format(self.name, alias))
+
+    def fetch(self, **kwargs):
+        return self.db.iquery('scan({})'.format(self),
+                              fetch=True,
+                              **kwargs)
+
+    def head(self, n=5, **kwargs):
+        """Similar to ``pandas.DataFrame.head``. Makes use of the ``limit``
+        operator, if available.
+
+        """
+        if 'limit' in self.db.operators:
+            return self.db.iquery('limit({}, {})'.format(self, n),
+                                  fetch=True,
+                                  **kwargs)
+        else:
+            warnings.warn(
+                '"limit" operator not available. ' +
+                'Fetching the entire array. ' +
+                'See https://github.com/Paradigm4/limit.')
+            return self.fetch(**kwargs)[:n]
+
+    def schema(self):
+        return Schema.fromstring(
+            self.db.iquery_readlines("show({})".format(self))[0])
 
 
 class ArrayExp(object):
@@ -557,13 +697,21 @@ class Operator(object):
             arg_fmt = '{}'.format(arg)
 
             # Special case: quote string argument if not quoted
-            if (self.name in string_args[pos] and
+            if (pos < len(string_args) and
+                    self.name in string_args[pos] and
                     arg and
                     arg_fmt[0] != "'" and
                     arg_fmt[-1] != "'"):
 
                 arg_fmt = "'{}'".format(arg_fmt)
 
+            # Special case: -- - show - --
+            if (pos == 0 and
+                    self.name == 'show' and
+                    (isinstance(arg, Operator) or
+                     len(self.args) > 1)):
+                arg_fmt = "'{}'".format(arg_fmt.replace("'", "\\'"))
+
             # Add to arguments list
             args_fmt.append(arg_fmt)
 
@@ -572,6 +720,20 @@ class Operator(object):
     def __call__(self, *args, **kwargs):
         """Returns self for lazy expressions. Executes immediate expressions.
         """
+        # Propagate "upload_data" and "upload_schema" from previous
+        # operators
+        for arg in args:
+            if isinstance(arg, Operator) and (
+                    arg.upload_data is not None or
+                    arg.upload_schema is not None):
+                if self.upload_data is not None or (self.upload_schema
+                                                    is not None):
+                    raise NotImplementedError(
+                        'Queries with multiple "upload_data" or ' +
+                        '"upload_schema" arguments are not supported')
+                self.upload_data = arg.upload_data
+                self.upload_schema = arg.upload_schema
+
         self.args.extend(args)
 
         # Special case: -- - create_array - --
@@ -598,7 +760,7 @@ class Operator(object):
                     try:
                         self.upload_schema = Schema.fromdtype(
                             self.upload_data.dtype)
-                    except:
+                    except Exception:
                         # Might fail if the dtype contains
                         # objects. The same type mapping is attempted
                         # later in iquery, but there the exception is
@@ -611,32 +773,53 @@ class Operator(object):
                         ln >= 1):
                     try:
                         self.upload_schema = Schema.fromstring(args[0])
-                    except:
+                    except Exception:
                         # Fails if the argument is an array name
                         pass
 
-            # Set defaults if arguments are missing
+            # Set required arguments if missing
+            # Check if "input_file" is present (2nd argument)
+            if ln < 2:
+                # Check if "existing_array|anonymous_schema"
+                # is present (1st argument)
+                if ln < 1:
+                    self.args.append('{sch}')  # anonymous_schema
+                self.args.append("'{fn}'")     # input_file
+
+            # Set optional arguments if missing and necessary
             # Check if "format" is present (4th argument)
-            if ln < 4:
-                # Check if "instance_id" is present (3rd argument)
+            if ln < 4 and self.upload_data is not None:
+                # Check if "instance_id" is present (3nd argument)
                 if ln < 3:
-                    # Check if "input_file" is present (2nd argument)
-                    if ln < 2:
-                        # Check if "existing_array|anonymous_schema"
-                        # is present (1st argument)
-                        if ln < 1:
-                            self.args.append('{sch}')  # anonymous_schema
-                        self.args.append("'{fn}'")     # input_file
-                    self.args.append(0)                # instance_id
-                self.args.append("'{fmt}'")            # format
+                    self.args.append(0)        # instance_id
+                self.args.append("'{fmt}'")    # format
 
         # Special case: -- - store - --
-        elif self.name == 'store' and len(self.args) < 2:
-            # Set "named_array"
-            self.args.append(self.db.next_array_name())
-            # Garbage collect (if not specified)
-            if 'gc' not in kwargs.keys():
-                kwargs['gc'] = True
+        elif self.name == 'store':
+            if len(self.args) < 2:
+                # Set "named_array"
+                self.args.append(self.db.next_array_name())
+                # Garbage collect (if not specified)
+                if 'gc' not in kwargs.keys():
+                    kwargs['gc'] = True
+            # If temp=True in kwargs, create a temporary array first
+            if 'temp' in kwargs.keys() and kwargs['temp'] is True:
+                # Get the schema of the new array
+                try:
+                    new_schema = Schema.fromstring(
+                        self.db.iquery_readlines(
+                            "show('{}', 'afl')".format(
+                                str(self.args[0]).replace("'", "\\'")))[0])
+                except requests.HTTPError as e:
+                    e.args = (
+                        '"temp=True" not supported for complex queries\n' +
+                        e.args[0],
+                    )
+                    raise
+                # Set array name
+                new_schema.name = self.args[1]
+                # Create temporary array
+                self.db.iquery('create temp array {}'.format(new_schema))
 
         # Lazy or hungry
         if self.is_lazy:        # Lazy
@@ -680,14 +863,23 @@ class Operator(object):
     def __dir__(self):
         return self._dir
 
-    def fetch(self, atts_only=False, as_dataframe=False):
+    def __mod__(self, alias):
+        """Overloads ``%`` operator to add support for aliasing"""
+        return Array(self.db, '{} as {}'.format(self, alias))
+
+    def fetch(self, **kwargs):
         if self.is_lazy:
             return self.db.iquery(str(self),
                                   fetch=True,
-                                  atts_only=atts_only,
-                                  as_dataframe=as_dataframe,
                                   upload_data=self.upload_data,
-                                  upload_schema=self.upload_schema)
+                                  upload_schema=self.upload_schema,
+                                  **kwargs)
+
+    def schema(self):
+        if self.is_lazy:
+            return Schema.fromstring(
+                self.db.iquery_readlines(
+                    "show('{}', 'afl')".format(self))[0])
 
 
 connect = DB
